@@ -311,7 +311,7 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
         else
             m_combo->lineEdit()->setText( "/" );*/
 
-	QVBoxLayout *v1 = new QVBoxLayout( vb, 3, 3 );
+	QVBoxLayout *v1 = new QVBoxLayout( 0, 3, 3 );
 	v1->addWidget( browserComb );
 	fileBrowser = new KDirOperator( KURL("/"), vb );
 	fileBrowser->setupMenu( KDirOperator::SortActions|KDirOperator::NavActions|KDirOperator::ViewActions );
@@ -387,6 +387,11 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	layout->addMultiCellWidget(coverFrame, 0, 2, 0, 0);
 
 	QHBoxLayout *h1 = new QHBoxLayout();
+	searchBtn = new QToolButton( playlist );
+	searchBtn->setAutoRaise( true );
+	QToolTip::add( searchBtn, i18n("Search"));
+	searchBtn->setIconSet( KGlobal::iconLoader()->loadIconSet("locationbar_erase", KIcon::Small) );
+	h1->addWidget( searchBtn );
 	QLabel* filterLabel = new QLabel(i18n("Filter") + ":", playlist);
 	h1->addWidget(filterLabel);
 	m_playlistFilter = new KLineEdit(playlist);
@@ -437,6 +442,7 @@ PlayList::PlayList( QWidget* parent, QObject *objParent, const char *name ) : Ka
 	connect(fileBrowser, SIGNAL(urlEntered( const KURL& )), this, SLOT(browserURLChanged( const KURL& )));
 	connect(fileBrowser, SIGNAL(fileSelected(const KFileItem*)), this, SLOT(fileSelected(const KFileItem*)));
 	connect(m_playlistFilter, SIGNAL(textChanged(const QString&)), this, SLOT(slotFindText(const QString&)));
+	connect( searchBtn, SIGNAL(clicked()), this, SLOT(resetSearch()) );
 	connect(m_playlistSelector, SIGNAL(returnPressed(const QString&)), this, SLOT(slotNewPlaylistName(const QString&)));
 	connect(m_playlistSelector, SIGNAL(activated(int)),this, SLOT(slotPlaylistActivated(int)));
 	connect(m_list, SIGNAL(dropped(QDropEvent*, QListViewItem*)), this, SLOT(slotDropEvent(QDropEvent*, QListViewItem*)));
@@ -2068,6 +2074,12 @@ void PlayList::exportPLSPlaylist(const QString& pl)
 	file.close();
 }
 
+void PlayList::resetSearch()
+{
+	m_playlistFilter->clear();
+	slotFindText( "" );
+}
+
 void PlayList::slotFindText(const QString& text)
 {
 	if (text == i18n("Filter")) return;
