@@ -1,22 +1,26 @@
-/***************************************************************************
-                          	dvbcam.cpp
-                             -------------------
-    copyright            : (C) 2006 by Christoph Pfister
-    email                :  christophpfister@gmail.com
- ***************************************************************************
-	code based on ca_zap (LGPL)
-
-	Copyright (C) 2004, 2005 Manu Abraham (manu@kromtek.com)
-	Copyright (C) 2006 Andrew de Quincey (adq_dvb@lidskialf.net)
-
-***************************************************************************
- *                                                                         *
- *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
- *                                                                         *
- ***************************************************************************/
+/*
+ * dvbcam.cpp
+ *
+ * Copyright (C) 2006 Christoph Pfister <christophpfister@gmail.com>
+ *
+ * code based on ca_zap (LGPL)
+ * Copyright (C) 2004, 2005 Manu Abraham (manu@kromtek.com)
+ * Copyright (C) 2006 Andrew de Quincey (adq_dvb@lidskialf.net)
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ */
 
 #include <unistd.h>
 #include <stdio.h>
@@ -39,8 +43,6 @@
 
 #include "dvbcam.h"
 
-
-
 class DvbCamCamHandler
 {
 public:
