@@ -1,7 +1,7 @@
 /*
  * dvbpanel.cpp
  *
- * Copyright (C) 2004-2006 Christophe Thommeret <hftom@free.fr>
+ * Copyright (C) 2004-2007 Christophe Thommeret <hftom@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,9 +50,9 @@
 #include "dvbpanel.h"
 #include "channeldesc.h"
 #include "dvbstream.h"
-#include "dvbevents.h"
 #include "kevents.h"
 #include "broadcasteditor.h"
+#include "channeleditor.h"
 
 #define CHANICONSIZE 28
 
@@ -80,6 +80,7 @@ void DIconViewItem::dropped( QDropEvent *e, const QValueList<QIconDragItem> & )
 
 DListView::DListView( QWidget *parent ) : KListView( parent )
 {
+	visibleItems = 0;
 }
 
 
@@ -148,6 +149,9 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	configBtn = new QToolButton( panel );
 	configBtn->setAutoRaise( true );
 	QToolTip::add( configBtn, i18n("DVB settings"));
+	recallBtn = new QToolButton( panel );
+	recallBtn->setAutoRaise( true );
+	QToolTip::add( recallBtn, i18n("Recall"));
 	QHBoxLayout *h1 = new QHBoxLayout();
 	h1->addItem( new QSpacerItem( 1, 1, QSizePolicy::Preferred, QSizePolicy::Minimum ) );
 	h1->addWidget( channelsBtn );
@@ -155,9 +159,22 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	h1->addWidget( infoBtn );
 	h1->addWidget( osdBtn );
 	h1->addWidget( configBtn );
+	h1->addWidget( recallBtn );
 	h1->addItem( new QSpacerItem( 1, 1, QSizePolicy::Preferred, QSizePolicy::Minimum ) );
 	vb->addLayout( h1 );
 
+	h1 = new QHBoxLayout();
+	searchBtn = new QToolButton( panel );
+	searchBtn->setAutoRaise( true );
+	QToolTip::add( searchBtn, i18n("Search channel(s)"));
+	QLabel* filterLabel = new QLabel( i18n("Filter") + ":", panel );
+	searchLE = new KLineEdit( panel );
+	searchLE->setFocusPolicy( QWidget::ClickFocus );
+	h1->addWidget( searchBtn );
+	h1->addWidget( filterLabel );
+	h1->addWidget( searchLE );
+	vb->addLayout( h1 );
+
 	channelsCb = new DListView( panel );
 	channelsCb->setHScrollBarMode( QListView::AlwaysOff );
 	//channelsCb->setPaletteBackgroundColor( QColor(255,255,200) );
@@ -230,6 +247,8 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	osdBtn->setIconSet( icon->loadIconSet("info", KIcon::Small) );
 	channelsBtn->setIconSet( icon->loadIconSet("kdvbtv", KIcon::Small) );
 	configBtn->setIconSet( icon->loadIconSet("configure", KIcon::Small) );
+	recallBtn->setIconSet( icon->loadIconSet("reload", KIcon::Small) );
+	searchBtn->setIconSet( icon->loadIconSet("locationbar_erase", KIcon::Small) );
 
 	setXMLFile("kaffeinedvb.rc");
 	setupActions();
@@ -237,6 +256,7 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	connect( audioComb, SIGNAL(activated(int)), this, SLOT(audioSelected(int)) );
 	connect( this, SIGNAL(zap(ChannelDesc*)), SLOT(dvbZap(ChannelDesc*)) );
 	connect( configBtn, SIGNAL(clicked()), this, SLOT(showConfigDialog()));
+	connect( recallBtn, SIGNAL(clicked()), this, SLOT(recallZap()));
 
 	connect( recordBtn, SIGNAL(clicked()), this, SLOT(setRecord()) );
 	connect( broadcastBtn, SIGNAL(clicked()), this, SLOT(setBroadcast()) );
@@ -244,6 +264,8 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 	connect( channelsBtn, SIGNAL(clicked()), this, SLOT(scanner()) );
 	connect( dateBtn, SIGNAL(clicked()), this, SLOT(showTimers()) );
 	connect( osdBtn, SIGNAL(clicked()), this, SLOT(dvbOSD()));
+	connect( searchBtn, SIGNAL(clicked()), this, SLOT(resetSearch()) );
+	connect(searchLE, SIGNAL(textChanged(const QString&)), this, SLOT(searchChannel(const QString&)));
 
 	connect( &timersTimer, SIGNAL(timeout()), this, SLOT(checkTimers()) );
 	connect( &osdTimer, SIGNAL(timeout()), this, SLOT(resetOSD()) );
@@ -256,7 +278,9 @@ DvbPanel::DvbPanel( QWidget *parent, QObject *objParent, const char *name ) : Ka
 
 	updown = 1;
 	autocount = 0;
-        delete icon;
+	delete icon;
+
+	events.loadEpg();
 }
 
 
@@ -303,6 +327,7 @@ void DvbPanel::channelPopup( QListViewItem *it, const QPoint &pos, int col )
 
 	QPopupMenu *pop = new QPopupMenu();
 	pop->insertItem( i18n("Select icon..."), 1 );
+	pop->insertItem( i18n("Edit..."), 2 );
 	i = 0;
 	i = pop->exec( QCursor::pos() );
 	switch ( i ) {
@@ -329,10 +354,58 @@ void DvbPanel::channelPopup( QListViewItem *it, const QPoint &pos, int col )
 			pix.save( dvbConfig->dvbConfigIconsDir+name, "PNG" );
 			fillChannelList();
 			break;
+		case 2:
+			editChannel( name );
+			break;
+
 	}
 	delete pop;
 }
 
+bool DvbPanel::editChannel( QString &name )
+{
+	int j;
+	ChannelDesc *chan=0;
+	QStringList list;
+	QPixmap pix;
+
+	for ( j=0; j<(int)channels.count(); j++ ) {
+		chan = channels.at(j);
+		if ( chan->name==name ) {
+			j = -1;
+			list.append( chan->tp.source );
+			break;
+		}
+	}
+	if ( j==-1 ) {
+		ChannelEditor dlg( list, false, chan, &channels, mainWidget );
+		int ret = dlg.exec();
+		if ( ret==ChannelEditor::Accepted ) {
+			pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
+			if ( !pix.isNull() )
+				chan->pix = pix;
+			else {
+				if ( chan->type==1 ) {
+					if ( chan->fta )
+						pix = tvcPix;
+					else
+						pix = tvPix;
+				}
+				else {
+					if ( chan->fta )
+						pix = racPix;
+					else
+						pix = raPix;
+				}
+			}
+			fillChannelList( chan );
+			return true;
+		}
+	}
+	return false;
+}
+
+
 
 
 void DvbPanel::getTargets( QStringList &uiNames, QStringList &iconNames, QStringList &targetNames )
@@ -362,12 +435,13 @@ bool DvbPanel::execTarget( const QString &target )
 
 void DvbPanel::setupActions()
 {
-	new KAction(i18n("OSD Next Channel"), "next", CTRL + SHIFT + Key_W, this, SLOT(dvbOSDNext()), actionCollection(), "dvb_browse_next");
-	new KAction(i18n("OSD Previous Channel"), "previous", CTRL + SHIFT + Key_Q, this, SLOT(dvbOSDPrev()), actionCollection(),"dvb_browse_prev");
-	new KAction(i18n("OSD Zap"), "ok", CTRL + SHIFT + Key_E, this, SLOT(dvbOSDZap()), actionCollection(), "dvb_browse_zap");
-	new KAction(i18n("OSD Next Event"), "down", CTRL + SHIFT + Key_S, this, SLOT(dvbOSDAdvance()), actionCollection(), "dvb_browse_advance");
-	new KAction(i18n("OSD Previous Event"), "up", CTRL + SHIFT + Key_A, this, SLOT(dvbOSDRetreat()), actionCollection(), "dvb_browse_retreat");
+	new KAction(i18n("OSD Next Channel"), "next", CTRL+SHIFT+Key_W, this, SLOT(dvbOSDNext()), actionCollection(), "dvb_browse_next");
+	new KAction(i18n("OSD Previous Channel"), "previous", CTRL+SHIFT+Key_Q, this, SLOT(dvbOSDPrev()), actionCollection(),"dvb_browse_prev");
+	new KAction(i18n("OSD Zap"), "ok", CTRL+SHIFT+Key_E, this, SLOT(dvbOSDZap()), actionCollection(), "dvb_browse_zap");
+	new KAction(i18n("OSD Next Event"), "down", CTRL+SHIFT+Key_S, this, SLOT(dvbOSDAdvance()), actionCollection(), "dvb_browse_advance");
+	new KAction(i18n("OSD Previous Event"), "up", CTRL+SHIFT+Key_A, this, SLOT(dvbOSDRetreat()), actionCollection(), "dvb_browse_retreat");
 	new KAction(i18n("Instant Record"), "filesave", 0, this, SLOT(setRecord()), actionCollection(), "dvb_instant_record");
+	new KAction(i18n("Recall"), "reload", CTRL+SHIFT+Key_Z, this, SLOT(recallZap()), actionCollection(), "dvb_recall");
 	new KAction(i18n("Show OSD"), "info", Key_O, this, SLOT(dvbOSD()), actionCollection(), "dvb_show_osd");
 	new KAction(i18n("EPG..."), "view_text", Key_G, this, SLOT(showEvents()), actionCollection(), "dvb_show_epg");
 	new KAction(i18n("Timers..."), "date", Key_T, this, SLOT(showTimers()), actionCollection(), "dvb_show_timers");
@@ -392,6 +466,45 @@ QWidget *DvbPanel::inputMainWidget()
 
 
 
+void DvbPanel::searchChannel( const QString &text )
+{
+	int i, c=-1;
+	QListViewItemIterator it( channelsCb );
+	if ( !it.current() )
+		return;
+	for ( i=0; i<channelsCb->columns(); i++ ) {
+		if ( channelsCb->columnText(i)==i18n("Name") ) {
+			c = i;
+			break;
+		}
+	}
+	if ( c==-1 )
+		return;
+
+	channelsCb->visibleItems = 0;
+
+	while ( it.current() ) {
+		if ( text.isEmpty() || it.current()->text( c ).contains( text, false ) )	{
+			it.current()->setVisible(true);
+			++channelsCb->visibleItems;
+		}
+		else {
+			it.current()->setVisible(false);
+		}
+		++it;
+	}
+}
+
+
+
+void DvbPanel::resetSearch()
+{
+	searchLE->clear();
+	searchChannel( "" );
+}
+
+
+
 void DvbPanel::channelNumberChanged( QListViewItem *it )
 {
 	int i, j;
@@ -519,10 +632,11 @@ void DvbPanel::moveChannel( const QString &cat, const QString &name )
 
 void DvbPanel::dumpEvents()
 {
-	DVBevents *events=0;
-	EventSourceList *slist;
-	EventTsidList *tlist;
-	EventDesc *desc=0;
+	EventSource *esrc;
+	EventTsid *et;
+	EventSid *es;
+	QPtrList<EventDesc> *esEvents;
+	EventDesc *desc;
 	int i, k, j, m, n;
 	QString s;
 
@@ -532,15 +646,19 @@ void DvbPanel::dumpEvents()
 		QTextStream tt( &f );
 		tt << "Saved: "+QDateTime::currentDateTime().toString( "dd-MM-yyyy hh:mm:ss" )+"\n";
 		k= 0;
-		for( i=0; i<(int)dvb.count(); i++ ) {
-			events = dvb.at(i)->dvbEvents;
-			for ( m=0; m<(int)events->srcList.count(); m++ ) {
-				slist = events->srcList.at(m);
-				tt << slist->source+QString(" : %1 TS with events.\n").arg(slist->tsidList.count());
-				for ( n=0; n<(int)slist->tsidList.count(); n++ ) {
-					tlist = slist->tsidList.at(n);
-					tt << QString("TSID %1 : %2 events.\n").arg(tlist->tsid).arg(tlist->events.count());
-					k+= tlist->events.count();
+		for( i=0; i<events.getNSource(); i++ ) {
+			if ( !(esrc=events.getNEventSource( i )) )
+				continue;
+			tt << esrc->getSource()+QString(" : %1 TS with events.\n").arg( esrc->getNTsid() );
+			for ( m=0; m<esrc->getNTsid(); m++ ) {
+				if ( !(et=esrc->getNEventTsid( m )) )
+					continue;
+				tt << QString("TSID %1 : %2 services with events\n").arg( et->getTsid() ).arg( et->getNSid() );
+				for ( n=0; n<et->getNSid(); n++ ) {
+					if ( !(es=et->getNEventSid( n )) )
+						continue;
+					tt << QString("     SID %1 : %2 events\n").arg( es->getSid() ).arg( es->getNDesc() );
+					k+= es->getNDesc();
 				}
 			}
 		}
@@ -548,15 +666,20 @@ void DvbPanel::dumpEvents()
 		fprintf( stderr, "Number of events : %d\n", k );
 		tt << "-------------------------\n";
 		tt << "\n";
-		for( i=0; i<(int)dvb.count(); i++ ) {
-			events = dvb.at(i)->dvbEvents;
-			events->mutex.lock();
-			for ( m=0; m<(int)events->srcList.count(); m++ ) {
-				slist = events->srcList.at(m);
-				for ( n=0; n<(int)slist->tsidList.count(); n++ ) {
-					tlist = slist->tsidList.at(n);
-					for ( j=0; j<(int)tlist->events.count(); j++ ) {
-						desc = tlist->events.at(j);
+		for( i=0; i<events.getNSource(); i++ ) {
+			if ( !(esrc=events.getNEventSource( i )) )
+				continue;
+			for ( m=0; m<esrc->getNTsid(); m++ ) {
+				if ( !(et=esrc->getNEventTsid( m )) )
+					continue;
+				for ( n=0; n<et->getNSid(); n++ ) {
+					if ( !(es=et->getNEventSid( n )) )
+						continue;
+					esEvents = es->getEvents();
+					es->lock();
+					for ( j=0; j<(int)esEvents->count(); j++ ) {
+						if ( !(desc=esEvents->at( j )) )
+							continue;
 						tt << "Source: "+desc->source+"\n";
 						tt << QString("Network ID: %1\n").arg(desc->nid);
 						tt << QString("Transport Stream ID: %1\n").arg(desc->tsid);
@@ -580,9 +703,9 @@ void DvbPanel::dumpEvents()
 						tt << s+"\n";
 						tt << "\n";
 					}
+					es->unlock();
 				}
 			}
-			events->mutex.unlock();
 		}
 		f.close();
 	}
@@ -737,36 +860,25 @@ void DvbPanel::dvbOSDRetreat()
 void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* = 0 */)
 {
 	QStringList list;
-	DVBevents *events;
+	EventSource *esrc;
+	EventSid *es;
+	QPtrList<EventDesc> *esEvents;
 	EventDesc *desc;
-	EventSourceList *slist=0;
-	EventTsidList *tlist=0;
-	QPtrList<EventDesc> *currentEvents=0;
 	QString s;
-	int i=0, j, k;
+	int i, j, k;
 	int first=0;
 
-	if ( !d ) {
-		//fprintf(stderr,"DVBOSD : d==null\n");
+	if ( !d || liveChannel.name.isEmpty() )
 		return;
-	}
-
-	if ( liveChannel.name.isEmpty() ) {
-		//fprintf(stderr,"DVBOSD : channelName==empty\n");
-		return;
-	}
 
 	if ( osdMode==2 ) {
 		osdTimer.stop();
 		osdMode = 0;
 		list.append( "STOP" );
 		emit showDvbOSD( s.setNum( liveChannel.num )+" - "+liveChannel.name, list );
-		//fprintf(stderr,"DVBOSD : mode==2\n");
 		return;
 	}
 
-	events = d->dvbEvents;
-
 	if ( osdMode==1 )
 		osdTimer.stop();
 
@@ -792,47 +904,16 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 	browseDvbTimeShift = myshift;
 
 	k = 0;
-	int myShifted = 0;
 
-	events->mutex.lock();
-	for ( j=0; j<(int)events->srcList.count(); j++ ) {
-		if ( events->srcList.at(j)->source==liveChannel.tp.source ) {
-			slist = events->srcList.at(j);
-			break;
-		}
-	}
-	if ( !slist ) {
-		//fprintf(stderr,"DVBOSD : slist==null\n");
-		goto end;
-	}
-	else {
-		//fprintf(stderr,"DVBOSD : liveChannel.tp.source==%s\n",liveChannel.tp.source.ascii());
-	}
+	esrc = events.getEventSource( liveChannel.tp.source );
+	es = esrc->getEventSid( liveChannel.tp.tsid, liveChannel.sid );
+	es->lock();
+	esEvents = es->getEvents();
 
-	for ( j=0; j<(int)slist->tsidList.count(); j++ ) {
-		tlist = slist->tsidList.at(j);
-		if ( tlist->tsid==liveChannel.tp.tsid ) {
-			currentEvents = &tlist->events;
-			break;
-		}
-	}
-	if ( !currentEvents ) {
-		//fprintf(stderr,"DVBOSD : currentEvents==null\n");
-		goto end;
-	}
-
-	for ( j=0; j<(int)currentEvents->count(); j++ ) {
-		desc = currentEvents->at(j);
-
-		//if ( desc->tid>0x4f ) continue;
-		if ( desc->source!=liveChannel.tp.source || desc->sid!=liveChannel.sid || desc->tsid!=liveChannel.tp.tsid )
+	for ( j=myshift; j<(myshift+2); j++ ) {
+		desc = esEvents->at( j );
+		if ( !desc )
 			continue;
-		//if ( desc->running!=4 && desc->running!=1 ) continue;
-
-		if (myShifted < myshift) {
-			myShifted++;
-			continue;
-		}
 
 		if ( !desc->title.isEmpty() ) {
 			s = desc->startDateTime.toString( "hh:mm" );
@@ -875,8 +956,8 @@ void DvbPanel::dvbOSD(ChannelDesc liveChannel, DvbStream *d, int timeShift /* =
 		list.append( s );
 		break;
 	}
-end:
-	events->mutex.unlock();
+	es->unlock();
+
 	if ( osdMode==0 )
 		osdTimer.start( 5000, true );
 	osdMode++;
@@ -938,7 +1019,7 @@ void DvbPanel::setConfig()
 	}
 
 	for ( i=0; i<(int)dvbConfig->devList.count(); i++ ) {
-		d = new DvbStream( dvbConfig->devList.at(i), dvbConfig->defaultCharset );
+		d = new DvbStream( dvbConfig->devList.at(i), dvbConfig->defaultCharset, &events );
 		d->setPlug( plug );
 		dvb.append( d );
 		connect( d, SIGNAL(shifting(bool)), this, SLOT(setShiftLed(bool)) );
@@ -969,8 +1050,9 @@ void DvbPanel::setConfig()
 	getChannelList();
 	rtp = new Ts2Rtp();
 	rtp->setSocket( dvbConfig->broadcastAddress, dvbConfig->broadcastPort, dvbConfig->senderPort );
-	cleaner = new Cleaner( dvbConfig->shiftDir );
+	cleaner = new Cleaner( dvbConfig->shiftDir, dvbConfig->recordDir );
 	split->setSizes( dvbConfig->splitSizes );
+	recallChannel = dvbConfig->lastChannel;
 }
 
 
@@ -997,7 +1079,7 @@ loop:
 	if ( ret==DvbConfigDialog::Rejected )
 		return;
 	rtp->setSocket( dvbConfig->broadcastAddress, dvbConfig->broadcastPort, dvbConfig->senderPort );
-	cleaner->setPath( dvbConfig->shiftDir );
+	cleaner->setPaths( dvbConfig->shiftDir, dvbConfig->recordDir );
 }
 
 
@@ -1026,15 +1108,16 @@ QPtrList<Transponder> DvbPanel::getSourcesStatus()
 
 
 
-void DvbPanel::fillChannelList()
+void DvbPanel::fillChannelList( ChannelDesc *ch )
 {
 	int i, j;
 	ChannelDesc *chan;
-	KListViewItem *it;
+	KListViewItem *it, *visible=0;
 	bool cont=false;
 	QPtrList<Transponder> trans = getSourcesStatus();
 	trans.setAutoDelete( true );
 
+	searchLE->clear();
 	channelsCb->clear();
 	for ( i=0; i<(int)channels.count(); i++ ) {
 		chan = channels.at(i);
@@ -1062,7 +1145,10 @@ void DvbPanel::fillChannelList()
 		else if ( currentCategory!="All" && chan->category!=currentCategory )
 			continue;
 		it = new KListViewItem( channelsCb, QString().sprintf("%04d", chan->num), chan->name, chan->tp.source );
+		if ( ch && ch==chan )
+			visible = it;
 		it->setDragEnabled( true );
+		chan->pix.load( dvbConfig->dvbConfigIconsDir+chan->name );
 		if ( !chan->pix.isNull() )
 			it->setPixmap( 1, chan->pix );
 		else {
@@ -1080,7 +1166,12 @@ void DvbPanel::fillChannelList()
 			}
 		}
 	}
+	channelsCb->visibleItems = channelsCb->childCount();
 	trans.clear();
+	if ( visible ) {
+		channelsCb->ensureItemVisible( visible );
+		channelsCb->setSelected( visible, true );
+	}
 }
 
 
@@ -1204,7 +1295,7 @@ void DvbPanel::checkTimers()
 					stopLive();
 					emit dvbStop();
 				}
-				if ( d->startTimer( chan, dvbConfig->recordDir, t ) ) {
+				if ( d->startTimer( chan, dvbConfig->recordDir, dvbConfig->sizeFile, t ) ) {
 					KProcess proc;
 					proc << QDir::homeDirPath()+"/bin/kaffeine_recording";
 					proc << "On";
@@ -1238,13 +1329,10 @@ void DvbPanel::setRecord()
 {
 	ChannelDesc curchan;
 	QString s="";
-	int ret, i, j, r=0;
+	int ret, i, r=0;
 	bool live=false;
 	RecTimer *t, *rt;
 	EventDesc *desc;
-	EventSourceList *slist=0;
-	EventTsidList *tlist=0;
-	QPtrList<EventDesc> *currentEvents=0;
 	DvbStream *d=0;
 
 	for ( i=0; i<(int)dvb.count(); i++ ) {
@@ -1259,34 +1347,9 @@ void DvbPanel::setRecord()
 		return;
 	}
 
-	d->dvbEvents->mutex.lock();
-	for ( j=0; j<(int)d->dvbEvents->srcList.count(); j++ ) {
-		if ( d->dvbEvents->srcList.at(j)->source==curchan.tp.source ) {
-			slist = d->dvbEvents->srcList.at(j);
-			break;
-		}
-	}
-	if ( slist ) {
-		for ( j=0; j<(int)slist->tsidList.count(); j++ ) {
-			tlist = slist->tsidList.at(j);
-			if ( tlist->tsid==curchan.tp.tsid ) {
-				currentEvents = &tlist->events;
-				break;
-			}
-		}
-		if ( currentEvents ) {
-			for ( j=0; j<(int)currentEvents->count(); j++ ) {
-				desc = currentEvents->at(j);
-				if ( desc->sid!=curchan.sid || desc->tsid!=curchan.tp.tsid )
-					continue;
-				if ( desc->running!=4 )
-					continue;
-				s = desc->title;
-				break;
-			}
-		}
-	}
-	d->dvbEvents->mutex.unlock();
+	desc = events.getEventDesc( curchan.tp.source, curchan.tp.tsid, curchan.sid, 0 );
+	if ( desc )
+		s = desc->title;
 
 	if ( s.isEmpty() )
 		s = curchan.name+"-"+QDateTime::currentDateTime().toString("yyyyMMdd-hhmmss");
@@ -1302,7 +1365,7 @@ void DvbPanel::setRecord()
 
 	ret = d->canStartTimer( live, &curchan );
 	if ( ret==0 ) {
-		if ( d->startTimer( &curchan, dvbConfig->recordDir, rt ) ) {
+		if ( d->startTimer( &curchan, dvbConfig->recordDir, dvbConfig->sizeFile, rt ) ) {
 			KProcess proc;
 			proc << QDir::homeDirPath()+"/bin/kaffeine_recording";
 			proc << "On";
@@ -1554,19 +1617,15 @@ loop:
 
 void DvbPanel::showEvents()
 {
-	int i;
-
-	for ( i=0; i<(int)dvb.count(); i++ )
-		dvb.at(i)->dvbEvents->doClean( false );
-	KEvents dlg( &channels, &dvb, mainWidget, dvbConfig->epgSize );
+	events.doClean( false );
+	KEvents dlg( &channels, &dvb, &events, mainWidget, dvbConfig->epgSize );
 	connect( &dlg, SIGNAL(addTimer(QString,QString,QDateTime,QTime)), this, SLOT(newTimer(QString,QString,QDateTime,QTime)) );
 	connect( &dlg, SIGNAL(zapTo(const QString &)), this, SLOT(channelSelected(const QString &)) );
 	dlg.exec();
 	dvbConfig->epgSize = dlg.size();
 	disconnect( &dlg, SIGNAL(addTimer(QString,QString,QDateTime,QTime)), this, SLOT(newTimer(QString,QString,QDateTime,QTime)) );
 	disconnect( &dlg, SIGNAL(zapTo(const QString &)), this, SLOT(channelSelected(const QString &)) );
-	for ( i=0; i<(int)dvb.count(); i++ )
-		dvb.at(i)->dvbEvents->doClean( true );
+	events.doClean( true );
 }
 
 
@@ -1679,6 +1738,13 @@ bool DvbPanel::trackNumber( int num, MRL& )
 
 
 
+void DvbPanel::recallZap()
+{
+	playNumChannel( recallChannel );
+}
+
+
+
 void DvbPanel::playNumChannel( int num )
 {
 	int j;
@@ -1970,12 +2036,18 @@ void DvbPanel::finalZap( DvbStream *d, ChannelDesc *chan, bool setAudio, int nap
 			if ( !fifoName.isEmpty() )
 				currentFifo = fifoName;
 		}
+		if ( channelsCb->visibleItems<5 ) {
+			resetSearch();
+		}
 	}
 	else {
 		isTuning = false;
 		emit dvbStop();
 	}
-	dvbConfig->lastChannel = chan->num;
+	if ( dvbConfig->lastChannel!=(int)chan->num ) {
+		recallChannel = dvbConfig->lastChannel;
+		dvbConfig->lastChannel = chan->num;
+	}
 	//QTimer::singleShot( 3000, this, SLOT(playLastChannel()) );
 }
 
@@ -2024,6 +2096,7 @@ void DvbPanel::stopLive()
 
 	for ( i=0; i<(int)dvb.count(); i++ ) {
 		if ( dvb.at(i)->hasLive() ) {
+			recallChannel = dvb.at(i)->getLiveChannel().num;
 			dvb.at(i)->preStopLive();
 			dvb.at(i)->stopLive( &chan );
 			break;
@@ -2580,6 +2653,7 @@ DvbPanel::~DvbPanel()
 		KService::Ptr service = KService::serviceByDesktopName( plugName );
 		KLibLoader::self()->unloadLibrary( service->library().ascii() );
 	}
+	events.saveEpg();
 }
 
 #include "dvbpanel.moc"
