@@ -1,7 +1,7 @@
 /*
  * dvbpanel.h
  *
- * Copyright (C) 2004-2006 Christophe Thommeret <hftom@free.fr>
+ * Copyright (C) 2004-2007 Christophe Thommeret <hftom@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,12 +34,14 @@
 #include <kled.h>
 #include <klistview.h>
 #include <kiconview.h>
+#include <klineedit.h>
 
 #include "scandialog.h"
 #include "dvbconfig.h"
 #include "krecord.h"
 #include "ts2rtp.h"
 #include "cleaner.h"
+#include "dvbevents.h"
 
 
 
@@ -56,6 +58,9 @@ class DListView : public KListView
 	Q_OBJECT
 public:
 	DListView( QWidget *parent );
+
+	int visibleItems;
+
 protected:
 	virtual QDragObject* dragObject();
 };
@@ -133,6 +138,7 @@ public slots:
 	void showConfigDialog();
 	void next();
 	void previous();
+	void recallZap();
 	void dvbOSD();
 
 	void dvbOSDNext();
@@ -153,7 +159,7 @@ private:
 	void setupActions();
 	bool getTimerList();
 	bool saveTimerList();
-	void fillChannelList();
+	void fillChannelList( ChannelDesc *ch=0 );
 	QPtrList<Transponder> getSourcesStatus();
 	void updateModeTimer( RecTimer *t );
 	DvbStream* getWorkingDvb( int mode, ChannelDesc *chan );
@@ -168,7 +174,7 @@ private:
 	DListView *channelsCb;
 	QComboBox *audioComb;
 	QToolButton *broadcastBtn, *recordBtn;
-	QToolButton *dateBtn, *infoBtn, *channelsBtn, *configBtn, *osdBtn;
+	QToolButton *dateBtn, *infoBtn, *channelsBtn, *configBtn, *osdBtn, *recallBtn;
 	KLed *shiftLed, *recordLed, *broadcastLed;
 	QTimer timersTimer;
 	DVBconfig *dvbConfig;
@@ -184,6 +190,9 @@ private:
 	bool isTuning;
 	QTimer tuningTimer, stopTuningTimer;
 	QTimer diskTimer;
+	KLineEdit *searchLE;
+	QToolButton *searchBtn;
+	int recallChannel;
 
 	int browseDvbStream;
 	int maxChannelNumber, minChannelNumber;
@@ -192,9 +201,14 @@ private:
 	KaffeineDvbPlugin *plug;
 	QString plugName;
 
+	EventTable events;
+
 private slots:
 
+	void searchChannel( const QString &text );
+	void resetSearch();
 	void checkTimers();
+	bool editChannel( QString &name );
 	void channelSelected( QListViewItem *it );
 	void channelSelected( const QString &name );
 	void channelPopup( QListViewItem *it, const QPoint &pos, int col );
