@@ -1,7 +1,7 @@
 /*
  * scandialog.cpp
  *
- * Copyright (C) 2004-2006 Christophe Thommeret <hftom@free.fr>
+ * Copyright (C) 2004-2007 Christophe Thommeret <hftom@free.fr>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -650,6 +650,32 @@ void ScanDialog::checkDuplicateName( ChannelDesc *chan )
 	chan->name = name;
 }
 
+bool ScanDialog::checkChannUpdate( ChannelDesc *chan )
+{
+	int i, j;
+	AudioPid a;
+	for ( i=0; i<(int)chandesc->count(); i++ ) {
+		if ( chan->tp.source == chandesc->at(i)->tp.source && chan->tp.tsid == chandesc->at(i)->tp.tsid && chan->sid == chandesc->at(i)->sid ) {
+			chan->category=chandesc->at(i)->category;
+			chan->num=chandesc->at(i)->num;
+			chan->name=chandesc->at(i)->name;
+			chan->fta=chandesc->at(i)->fta;
+			for ( j=0; j<chan->napid; j++ ) {
+				if ( chandesc->at(i)->apid[0].pid==chan->apid[j].pid ) {
+					if ( j>0 ) {
+						a = chan->apid[0];
+						chan->apid[0]=chan->apid[j];
+						chan->apid[j]=a;
+					}
+					break;
+				}
+			}
+			chandesc->replace(i,new ChannelDesc(*chan));
+			return false;
+		}
+	}
+	return true;
+}
 
 
 void ScanDialog::addSelected()
@@ -670,23 +696,24 @@ void ScanDialog::addSelected()
 			dit = (DListViewItem*)dit->nextSibling();
 			continue;
 		}
+		if(checkChannUpdate(chan)){
+			checkDuplicateName( chan );
 
-		checkDuplicateName( chan );
-
-		chan->num = chandesc->count()+1;
-		chandesc->append( new ChannelDesc( *chan ) );
-		it = new QListViewItem( channelsList, chan->name );
-		if ( chan->type==1 ) {
-			if ( chan->fta )
-				it->setPixmap( 0, tvcPix );
-			else
-				it->setPixmap( 0, tvPix );
-		}
-		else {
-			if ( chan->fta )
-				it->setPixmap( 0, racPix );
-			else
-				it->setPixmap( 0, raPix );
+			chan->num = chandesc->count()+1;
+			chandesc->append( new ChannelDesc( *chan ) );
+			it = new QListViewItem( channelsList, chan->name );
+			if ( chan->type==1 ) {
+				if ( chan->fta )
+					it->setPixmap( 0, tvcPix );
+				else
+					it->setPixmap( 0, tvPix );
+			}
+			else {
+				if ( chan->fta )
+					it->setPixmap( 0, racPix );
+				else
+					it->setPixmap( 0, raPix );
+			}
 		}
 		odit = dit;
 		dit = (DListViewItem*)dit->nextSibling();
@@ -775,25 +802,25 @@ void ScanDialog::addFiltered()
 			dit = (DListViewItem*)dit->nextSibling();
 			continue;
 		}
+		if(checkChannUpdate(chan)){
+			checkDuplicateName( chan );
 
-		checkDuplicateName( chan );
-
-		chan->num = chandesc->count()+1;
-		chandesc->append( new ChannelDesc( *chan ) );
-		it = new QListViewItem( channelsList, chan->name );
-		if ( chan->type==1 ) {
-			if ( chan->fta )
-				it->setPixmap( 0, tvcPix );
-			else
-				it->setPixmap( 0, tvPix );
+			chan->num = chandesc->count()+1;
+			chandesc->append( new ChannelDesc( *chan ) );
+			it = new QListViewItem( channelsList, chan->name );
+			if ( chan->type==1 ) {
+				if ( chan->fta )
+					it->setPixmap( 0, tvcPix );
+				else
+					it->setPixmap( 0, tvPix );
+			}
+			else {
+				if ( chan->fta )
+					it->setPixmap( 0, racPix );
+				else
+					it->setPixmap( 0, raPix );
+			}
 		}
-		else {
-			if ( chan->fta )
-				it->setPixmap( 0, racPix );
-			else
-				it->setPixmap( 0, raPix );
-		}
-
 		odit = dit;
 		dit = (DListViewItem*)dit->nextSibling();
 		delete odit;
